@@ -1,17 +1,22 @@
 from __future__ import annotations
 
+import os
+import signal
 import asyncio
-from threading import Thread
-import itertools
-from typing import TYPE_CHECKING, Dict, Optional, Any
+from queue import Empty
+from multiprocessing import Queue, Process
+from threading import Event
+from concurrent.futures import ThreadPoolExecutor
+from tblib import pickling_support
+from typing import Optional, Any
 
-from .trace import Trace
-from .utils import SubscribableDict, ThreadSafeAsyncioEvent, to_thread
-from .call import call_with_trace
-from . import script
+from .utils import SubscribableDict, to_thread
+from .process.run import run, RunArg, QueueCommands, QueueDone
+from .registrar import Registrar
+from .types import RunNo, TraceNo
+from .count import RunNoCounter
 
-if TYPE_CHECKING:
-    from .pdb.ci import PdbCommandInterface
+pickling_support.install()
 
 SCRIPT_FILE_NAME = "<string>"
 
@@ -28,12 +33,6 @@ class Machine:
             |    .-------------.   |
             |    |   Running   |   |
             |    '-------------'   |
-            |          | _exited() |
-            |          |           |
-            |          v           |
-            |    .-------------.   |
-            |    |   Exited    |   |
-            |    '-------------'   |
             |          | finish()  |
             |          |           |
             |          V           |
@@ -50,18 +49,22 @@ class Machine:
     """
 
     def __init__(self, statement: str, run_no_start_from=1):
-        self.registry = SubscribableDict()
+        filename = SCRIPT_FILE_NAME
+        self.registry = SubscribableDict[Any, Any]()
+        self._run_no = RunNo(run_no_start_from - 1)
+        self._run_no_count = RunNoCounter(run_no_start_from)
+        self._registrar = Registrar(self.registry)
 
-        self.registry["statement"] = statement
-        self.registry["script_file_name"] = SCRIPT_FILE_NAME
-        self.registry["run_no_count"] = itertools.count(
-            run_no_start_from
-        ).__next__
+        self.context = RunArg(
+            statement=statement, filename=filename, queue=self._registrar.queue
+        )
+
+        self._registrar.script_change(script=statement, filename=filename)
 
         self._lock_finish = asyncio.Condition()
         self._lock_close = asyncio.Condition()
 
-        self._state: State = Initialized(self.registry)
+        self._state: State = Initialized(self.context)
         self._state_changed()
 
     def __repr__(self):
@@ -69,7 +72,15 @@ class Machine:
         return f"<{self.__class__.__name__} {self.state_name!r}>"
 
     def _state_changed(self) -> None:
-        self.registry["state_name"] = self.state_name
+        self._registrar.state_change(self._state)
+        if self._state.name == "initialized":
+            self._run_no = self._run_no_count()
+            self.context["run_no"] = self._run_no
+            self._registrar.state_initialized(self._run_no)
+        elif self._state.name == "running":
+            self._registrar.run_start(self._run_no)
+        elif self._state.name == "finished":
+            self._registrar.run_end(state=self._state)
 
     @property
     def state_name(self) -> str:
@@ -83,40 +94,43 @@ class Machine:
         """Enter the running state"""
         self._state = self._state.run()
         self._state_changed()
-        self._task_exited = asyncio.create_task(self._exited())
 
-    async def _exited(self) -> None:
-        """Enter the exited state
+    def send_pdb_command(self, trace_id, command) -> None:
+        self._state.send_pdb_command(trace_id, command)
 
-        This method is scheduled as a task in run().
+    def interrupt(self) -> None:
+        self._state.interrupt()
 
-        Before changing the state to "exited", this method waits for the script
-        execution (in another thread) to exit.
-        """
-
-        self._state = await self._state.exited()
-        self._state_changed()
-
-    def send_pdb_command(self, thread_asynctask_id, command) -> None:
-        self._state.send_pdb_command(thread_asynctask_id, command)
+    def terminate(self) -> None:
+        self._state.terminate()
 
     async def finish(self) -> None:
         """Enter the finished state"""
-        await self._task_exited
         async with self._lock_finish:
             self._state = await self._state.finish()
             self._state_changed()
 
     def exception(self) -> Optional[Exception]:
-        return self._state.exception()
+        ret = self._state.exception()
+        return ret
 
     def result(self) -> Any:
         return self._state.result()
 
-    def reset(self, statement: Optional[str] = None) -> None:
+    def reset(
+        self,
+        statement: Optional[str] = None,
+        run_no_start_from: Optional[int] = None,
+    ) -> None:
         """Enter the initialized state"""
         if statement:
-            self.registry["statement"] = statement
+            self.context["statement"] = statement
+            self._registrar.script_change(
+                script=statement, filename=SCRIPT_FILE_NAME
+            )
+        if run_no_start_from is not None:
+            self._run_no_count = RunNoCounter(run_no_start_from)
+            # self._registrar.reset_run_no_count(run_no_start_from)
         self._state = self._state.reset()
         self._state_changed()
 
@@ -125,6 +139,7 @@ class Machine:
         async with self._lock_close:
             self._state = self._state.close()
             self._state_changed()
+            await to_thread(self._registrar.close)
             await to_thread(self.registry.close)
 
 
@@ -168,10 +183,6 @@ class State(ObsoleteMixin):
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    async def exited(self) -> State:
-        self.assert_not_obsolete()
-        raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
-
     async def finish(self) -> State:
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
@@ -182,12 +193,19 @@ class State(ObsoleteMixin):
 
     def close(self) -> State:
         self.assert_not_obsolete()
+        print(self.name)
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def send_pdb_command(self, trace_id: int, command: str) -> None:
+    def send_pdb_command(self, trace_id: TraceNo, command: str) -> None:
         del trace_id, command
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
+    def interrupt(self) -> None:
+        raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
+
+    def terminate(self) -> None:
+        raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
+
     def exception(self) -> Optional[Exception]:
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
@@ -200,33 +218,31 @@ class Initialized(State):
 
     Parameters
     ----------
-    registry : object
-        An instance of Registry.
+    context : dict
+        An instance of Context
 
     """
 
     name = "initialized"
 
-    def __init__(self, registry: SubscribableDict):
-        self.registry = registry
-        run_no = self.registry.get("run_no_count")()  # type: ignore
-        self.registry["run_no"] = run_no
+    def __init__(self, context: RunArg):
+        self._context = context
 
     def run(self):
         self.assert_not_obsolete()
-        running = Running(self.registry)
+        running = Running(self._context)
         self.obsolete()
         return running
 
     def reset(self):
         self.assert_not_obsolete()
-        initialized = Initialized(registry=self.registry)
+        initialized = Initialized(context=self._context)
         self.obsolete()
         return initialized
 
     def close(self):
         self.assert_not_obsolete()
-        closed = Closed(self.registry)
+        closed = Closed()
         self.obsolete()
         return closed
 
@@ -236,111 +252,52 @@ class Running(State):
 
     Parameters
     ----------
-    registry : object
-        An instance of Registry
+        An instance of Context
     """
 
     name = "running"
 
-    def __init__(self, registry: SubscribableDict):
-        self.registry = registry
-        self._event = ThreadSafeAsyncioEvent()
+    def __init__(self, context: RunArg):
+        self._context = context
+        self._q_commands: QueueCommands = Queue()
+        self._event = Event()
+        self._executor = ThreadPoolExecutor(max_workers=1)
+        self._f = self._executor.submit(self._run)
+        assert self._event.wait(2.0)
 
-        statement = self.registry.get("statement")
-
-        self._pdb_ci_map: Dict[int, PdbCommandInterface] = {}
-        trace = Trace(
-            registry=self.registry,
-            pdb_ci_map=self._pdb_ci_map,
+    def _run(self):
+        q_done: QueueDone = Queue()
+        self._p = Process(
+            target=run,
+            args=(self._context, self._q_commands, q_done),
+            daemon=True,
         )
-
-        self.loop = asyncio.get_running_loop()
-
-        def run():
-            code = statement
-            if isinstance(code, str):
-                try:
-                    code = compile(code, SCRIPT_FILE_NAME, "exec")
-                except BaseException as e:
-                    self._done(None, e)
-                    return
-            func = script.compose(code)
-            result = None
-            exception = None
-            try:
-                result = call_with_trace(func, trace)
-            except BaseException as e:
-                exception = e
-            self._done(result, exception)
-
-        self._thread = Thread(target=run, daemon=True)
-        self._thread.start()
-
-    def _done(self, result=None, exception=None):
-
-        if self.loop.is_closed():
-            # The exit is not being waited in the main thread, for example,
-            # exited() is not called.
-            return
-
-        self._exited = Exited(
-            self.registry,
-            thread=self._thread,
-            result=result,
-            exception=exception,
-        )
-
+        self._p.start()
+        self._event.set()
+        self._p.join()
         try:
-            self._event.set()
-        except RuntimeError:
-            # The event loop is closed.
-            pass
-
-    async def exited(self):
-        """return the exited state after the script exits."""
-        await self._event.wait()
-        self.obsolete()
-        return self._exited
-
-    def send_pdb_command(self, trace_id: int, command: str) -> None:
-        pdb_ci = self._pdb_ci_map[trace_id]
-        pdb_ci.send_pdb_command(command)
-
-
-class Exited(State):
-    """The state "exited", the script execution has exited
-
-    Parameters
-    ----------
-    registry : object
-        An instance of Registry
-    thread : object
-        The object of the thread in which the script was executed.
-        This thread is to be joined.
-    result : any
-        The result of the script execution, always None
-    exception : exception or None
-        The execution raised in the script execution if any. Otherwise
-        None
-    """
-
-    name = "exited"
-
-    def __init__(self, registry: SubscribableDict, thread, result, exception):
-        self.registry = registry
-        self._thread = thread
-        self._result = result
-        self._exception = exception
+            return q_done.get(timeout=0.1)
+        except Empty:
+            return None, None  # ret, exc
 
     async def finish(self):
         self.assert_not_obsolete()
-        await to_thread(self._thread.join)
-        finished = Finished(
-            self.registry, result=self._result, exception=self._exception
-        )
+        ret, exc = await to_thread(self._f.result)
+        self._executor.shutdown()
+        finished = Finished(self._context, result=ret, exception=exc)
         self.obsolete()
         return finished
 
+    def send_pdb_command(self, trace_id: TraceNo, command: str) -> None:
+        self._q_commands.put((trace_id, command))
+
+    def interrupt(self) -> None:
+        if self._p.pid:
+            os.kill(self._p.pid, signal.SIGINT)
+
+    def terminate(self) -> None:
+        self._p.terminate()
+
 
 class Finished(State):
     """The state "finished", the script execution has finished
@@ -349,8 +306,8 @@ class Finished(State):
 
     Parameters
     ----------
-    registry : object
-        An instance of Registry
+    context : dict
+        An instance of Context
     result : any
         The result of the script execution, always None
     exception : exception or None
@@ -360,11 +317,11 @@ class Finished(State):
 
     name = "finished"
 
-    def __init__(self, registry: SubscribableDict, result, exception):
+    def __init__(self, context: RunArg, result, exception):
         self._result = result
         self._exception = exception
 
-        self.registry = registry
+        self._context = context
 
     def exception(self):
         """Return the exception of the script execution
@@ -398,31 +355,22 @@ class Finished(State):
 
     def reset(self):
         self.assert_not_obsolete()
-        initialized = Initialized(registry=self.registry)
+        initialized = Initialized(context=self._context)
         self.obsolete()
         return initialized
 
     def close(self):
         self.assert_not_obsolete()
-        closed = Closed(self.registry)
+        closed = Closed()
         self.obsolete()
         return closed
 
 
 class Closed(State):
-    """The state "closed"
-
-    Parameters
-    ----------
-    registry : object
-        An instance of Registry
-    """
+    """The state "closed" """
 
     name = "closed"
 
-    def __init__(self, registry: SubscribableDict):
-        self.registry = registry
-
     def close(self):
         self.assert_not_obsolete()
         return self
