@@ -1,6 +1,6 @@
 import pytest
 
-from nextline.state import Closed
+from nextline.state import State, Closed
 
 from .base import BaseTestState
 
@@ -10,10 +10,10 @@ class TestClosed(BaseTestState):
     state_class = Closed
 
     @pytest.fixture()
-    def state(self, closed):
-        yield closed
+    def state(self, closed: State) -> State:
+        return closed
 
-    def test_close(self, state):
+    def test_close(self, state: State):
         # The same object should be returned no matter
         # how many times called.
         assert state is state.close()
